<?php
/* ffitem.php -- MyBB plugin for querying Final Fantasy Online item databases
  (based on my wowitem plugin)
  version 1.0.0, October 14th, 2010
  http://www.ravenguard.ca/ffitem/

  Copyright (C) 2009 Daniel Major

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

  Daniel Major dmajor@gmail.com

*/

// Disallow direct access to this file for security reasons
if(!defined("IN_MYBB"))
{
	die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}

define(FFITEM_HEADER, '<script type="text/javascript" src="http://static.yg.com/js/exsyndication.js"></script>');

$plugins->add_hook("parse_message", "ffitem_parse");

function ffitem_info()
{
	return array(
		"name"          => "FF Item Links",
		"description"   => "Queries online databases to display Final Fantasy Online item data in posts.",
		"website"       => "http://www.ravenguard.ca/ffitem/",
		"author"        => "Bakdu",
		"authorsite"    => "mailto:bakdu@ravenguard.ca",
		"version"       => "1.0.0",
		"guid"          => "fd211b5ec4e0615da65da31999cd59f2",
		"compatibility" => "14*,16*"
	);
}

function ffitem_install()
{
	global $mybb, $db;

	// create cache table
	$db->write_query("CREATE TABLE ".TABLE_PREFIX."ffitem (
		cache_key varchar(255) NOT NULL,
		fetched int unsigned NOT NULL,
		id int NOT NULL,
		name varchar(255) NOT NULL,
		icon_s varchar(255) NOT NULL,
		icon_m varchar(255) NOT NULL,
		icon_l varchar(255) NOT NULL,
		link varchar(255) NOT NULL,
		PRIMARY KEY (cache_key)
	);");

	// delete settings that were not uninstalled cleanly
	$db->delete_query('settings', "name LIKE 'ffitem_%'");
	$db->delete_query('settinggroups', "name = 'ffitem'");

	// add settings group
	$query = $db->simple_select('settinggroups', "MAX(disporder) AS max_disporder");
	$disporder = $db->fetch_field($query, 'max_disporder') + 1;
	$gid = $db->insert_query('settinggroups', array(
		'name' => 'ffitem',
		'title' => 'FF Item Links',
		'description' => $db->escape_string("Options to configure the FF Item Links plugin."),
		'disporder' => $disporder,
		'isdefault' => 0
	));

	// add settings
	$db->insert_query('settings', array(
		'name' => 'ffitem_db',
		'title' => 'Item Database',
		'description' => $db->escape_string("Choose the provider to query for item data.<br />\n<br />\nIf set to disabled, no more requests will be sent to external sites but the locally cached items will still show."),
		'optionscode' => "radio\nyg14=Yellow Gremlin (FFXIV)\nyg11=Yellow Gremlin (FFXI)\ndisabled=Disabled",
		'value' => 'yg14',
		'disporder' => 1,
		'gid' => $gid
	));

	$db->insert_query('settings', array(
		'name' => 'ffitem_lang',
		'title' => 'Language',
		'description' => $db->escape_string("Choose the language to query the database in."),
		'optionscode' => "radio\nen=English\nde=Deutsch\nfr=Fran&#231;ais\nja=&#26085;&#26412;&#35486;",
		'value' => 'en',
		'disporder' => 2,
		'gid' => $gid
	));

	$db->insert_query('settings', array(
		'name' => 'ffitem_class',
		'title' => 'Display Style',
		'description' => $db->escape_string("Choose the style to display links in."),
		'optionscode' => "radio\ntext=Text only\nyg-iconsmall=Text with small icon\nyg-iconmedium=Text with medium icon\nyg-iconlarge=Text with large icon\nyg-iconsmall yg-notext=Small icon\nyg-iconmedium yg-notext=Medium icon\nyg-iconlarge yg-notext=Large icon",
		'value' => 'text',
		'disporder' => 3,
		'gid' => $gid
	));

	$db->insert_query('settings', array(
		'name' => 'ffitem_cached',
		'title' => 'Enable Cache',
		'description' => $db->escape_string("If the cache is enabled, the item data will be stored in the database. This reduces the number of external requests and allows links to work during provider downtime but adds to the database size."),
		'optionscode' => "yesno",
		'value' => '1',
		'disporder' => 4,
		'gid' => $gid
	));

	$db->insert_query('settings', array(
		'name' => 'ffitem_cache_refresh',
		'title' => 'Cache Refresh',
		'description' => $db->escape_string("The number of minutes before the cached data is considered stale and an external request is made to refresh the data. Set to 0 to disable refreshing."),
		'optionscode' => "text",
		'value' => '1440',
		'disporder' => 5,
		'gid' => $gid
	));

	rebuild_settings();
}

function ffitem_is_installed()
{
	global $mybb, $db;

	return $db->table_exists('ffitem');
}

function ffitem_uninstall()
{
	global $mybb, $db;

	// remove settings
	$db->delete_query('settings', "name LIKE 'ffitem_%'");
	$db->delete_query('settinggroups', "name = 'ffitem'");

	rebuild_settings();

	// drop cache
	$db->drop_table('ffitem');
}

function ffitem_activate()
{
	require_once MYBB_ROOT.'/inc/adminfunctions_templates.php';
	find_replace_templatesets('headerinclude', '#{\$newpmmsg}#', FFITEM_HEADER . "\n{\$newpmmsg}");
}

function ffitem_deactivate()
{
	require_once MYBB_ROOT.'/inc/adminfunctions_templates.php';
	find_replace_templatesets('headerinclude', '#' . preg_quote(FFITEM_HEADER) . '\n#', '', 0);
}

function ffitem_parse($message)
{
	// the MyCode tag(s) to parse for, type => MyCode
	$ffitem_mycode = array(
		'item'      => 'item',
		'recipe'    => 'recipe',
		'skill'     => 'skill',
		'npc'       => 'npc',
		'quest'     => 'quest',
		'guildleve' => 'guildleve',
	);

	foreach ($ffitem_mycode as $type => $mycode)
	{
		$message = preg_replace('#\['.$mycode.'\](.*?)\[/'.$mycode.'\]#ie', "ffitem_code('".$type."','\$1')", $message);
	}
}

function ffitem_code($type, $data)
{
	global $mybb, $db;

	// fetch from cache or provider
	$stale = 0;
	$cache = ffitem_cache_query($type, $data);
	if ($cache)
	{
		$cache_refresh = intval($mybb->settings['ffitem_cache_refresh']) * 60;
		$stale = ($cache_refresh > 0) && ($cache['fetched'] < TIME_NOW - $cache_refresh);
	}
	if (!$cache || $stale)
	{
		eval("\$query = ffitem_" . $mybb->settings['ffitem_db'] . "_query(\$type, \$data);");
		if ($query && $mybb->settings['ffitem_cached'] == 1)
		{
			if ($stale)
			{
				$db->update_query('ffitem', array(
					'fetched' => $query['fetched'],
					'id' => $query['id'],
					'name' => $db->escape_string($query['name']),
					'icon_s' => $db->escape_string($query['icon_s']),
					'icon_m' => $db->escape_string($query['icon_m']),
					'icon_l' => $db->escape_string($query['icon_l']),
					'link' => $db->escape_string($query['link']),
				), "cache_key = '" . $db->escape_string($type . ':' . $data) . "'");
			}
			else
			{
				$db->insert_query('ffitem', array(
					'cache_key' => $db->escape_string($type . ':' . $data),
					'fetched' => $query['fetched'],
					'id' => $query['id'],
					'name' => $db->escape_string($query['name']),
					'icon_s' => $db->escape_string($query['icon_s']),
					'icon_m' => $db->escape_string($query['icon_m']),
					'icon_l' => $db->escape_string($query['icon_l']),
					'link' => $db->escape_string($query['link']),
				));
			}
		}
	}
	if (!$query) $query = $cache;

	// no cached data and error with provider, make up a null item
	if (!$query)
	{
		$query = array(
			'id'      => 0,
			'name'    => $data,
			'icon_s'  => '',
			'icon_m'  => '',
			'icon_l'  => '',
			'link'    => 'javascript:void(0);',
			'fetched' => TIME_NOW,
		);
	}

	// HTML is generated, customize here if desired
	$class = $mybb->settings['ffitem_class'] == 'text' ? '' : ' class="' . $mybb->settings['ffitem_class'] . '"';
	return '<a' . $class . ' href="' . $query['link'] . '">' . $query['name'] . '</a>';
}

function ffitem_cache_query($type, $data)
{
	global $mybb, $db;

	if ($mybb->settings['ffitem_cached'] == 1)
	{
		$query = $db->simple_select('ffitem', 'id, name, icon_s, icon_m, icon_l, link, fetched', "cache_key = '" . $db->escape_string($type . ':' . $data) . "'");
		$cache = $db->fetch_array($query);
		if (isset($cache['id']))
		{
			return $cache;
		}
	}

	return false;
}

function ffitem_disabled_query($type, $data)
{
	return false;
}

function ffitem_yg14_query($type, $data) { return ffitem_yg_query($type, $data, 'ffxiv'); }
function ffitem_yg11_query($type, $data) { return ffitem_yg_query($type, $data, 'ffxi'); }

function ffitem_yg_query($type, $data, $ygdb)
{
	global $mybb;

	if ($ygdb == 'ffxi' && $type != 'item') return false; // FFXI only supports 'item' type
	$query = ctype_digit($data) ? 'i=' . intval($data) : 's=' . rawurlencode($data);
	$lang = $mybb->settings['ffitem_lang'];
	$lang = $lang == 'en' ? '' : $lang . '/';
	$url = 'http://' . $ygdb . '.yg.com/' . $lang . 'api/lookup?' . $query . '&o=' . $type . '&t=xml';
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_TIMEOUT, 5);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	$response = curl_exec($ch);
	curl_close($ch);
	if ($response)
	{
		$xml = simplexml_load_string($response);
		if (!$xml->error) {
			$obj = $xml->object[0];
			if ($obj) {
				return array(
					'id'      => intval($obj->id),
					'name'    => ''.$obj->name,
					'icon_s'  => ''.$obj->icons->small,
					'icon_m'  => ''.$obj->icons->medium,
					'icon_l'  => ''.$obj->icons->large,
					'link'    => ''.$obj->url,
					'fetched' => TIME_NOW
				);
			}
		}
	}

	return false;
}

